use psxcdgen_ex::types::{layout::Layout, CDDesc, Data, Directory};
use tool_helper::Error;

fn populate() -> Result<CDDesc, Error> {
    let mut desc   = CDDesc::new();
    let     file   = Data::new("Planschi.jpg")?;
    let     file2  = Data::new("Wuff.png")?;
    let     folder = {
        let mut folder     = Directory::new("Sub")?;
        let     sub_folder = {
            let mut folder = Directory::new("SubSub")?;

            folder.add_data(Data::new("Blubb.bin")?);
            folder
        };

        folder.add_dir(sub_folder);
        folder.add_data(Data::new("Schwimm.jpg")?);
        folder.add_data(Data::new("Miau.png")?);
        folder
    };

    desc.root.add_dir(folder);
    desc.root.add_data(file);
    desc.root.add_data(file2);
    Ok(desc)
}

fn run_main() -> Result<(), Error> {
    let desc = populate()?;

    for element in desc.get_memory_layout().iter() {
        match element {
            Layout::SystemArea(_)                  => println!("SystemArea:"),
            Layout::PVD(_)                         => println!("PVD:"),
            Layout::Directory{name, properties: _} => println!("Dir: {}", name),
            Layout::Data(data)                     => println!("File: {}", data),
        }
    }
    Ok(())
}

fn main() {
    match run_main() {
        Ok(_)      => println!("\n<== Planschbecken ==>"),
        Err(error) => println!("{}", error)
    }
}